/*#============================================================================
 *# Copyright (C) 1991 Recital Corporation Inc.
 *# As an unpublished licenced proprietary work.
 *# All rights reserved worldwide.
 *#============================================================================
 *
 * 	MODULE		: rsi.h
 *
 * 	PURPOSE		: Recital/RSI constant definitions.
 *
 * 	AUTHOR		: Recital Corporation Inc.
 *
 * 	DATE		: 09-jul-1991
 *
 *===========================================================================*/
#ifndef RSI_H
#define RSI_H


/*-------------------*/
/* general constants */
/*-------------------*/
#ifndef MAX_RECORD_FIELDS
#define MAX_RECORD_FIELDS       256
#endif
#define RSI_PROTOCOL_TCPIP	1
#define RSI_PROTOCOL_DECNET	2
#define	MAX_RSI_PKTSIZE		(16*1024)
#define	MAX_RSI_METAPKTSIZE	128
#define MAX_RSI_ARGS		128
#define RSI_CURSOR_READONLY	0
#define RSI_CURSOR_UPDATE	1
#define RSI_CURSOR_INSERTONLY	2
#define MAX_SQL_NAME_LENGTH     80
#ifndef MAX_FILENAME_LENGTH
#define MAX_FILENAME_LENGTH     512
#endif
#define MAX_ACTIVE_WORKAREAS    100
#define MAX_STRING_LENGTH       512
#define MAX_NAME_LENGTH         10
#ifndef _WINDOWS
#ifndef O_RDWR
#define O_RDWR			2
#endif
#ifndef max
#define max(x, y) \
        (( (x) > (y) ) ? (x) : (y) )
#endif
#ifndef min
#define min(x, y) \
        (( (x) > (y) ) ? (y) : (x) )
#endif
#endif
#ifndef TRUE
#define TRUE                    1
#endif
#ifndef FALSE
#define FALSE                   0
#endif

/*-----------------------------------------------------------------------------
 *
 * Layout of an RSI packet :-
 *
 * 	+-------+
 * 	|	|		pkt_type[1]	     packet type
 * 	+-------+
 * 	|	|		pkt_info[8]	     packet secondary info
 *      .........
 *      |       |
 * 	+-------+
 * 	|	|		pkt_argc[4]	     packet arg count
 *	+-------+
 *	|	|----+		pkt_argv[pkt_argc*4]  offsets of args in packet
 *	.........    |
 *	|	|    |
 *	.........    |
 *	|	|    |
 *      +-------+    |
 * 	|	|<---+		pkt_arglen[4]	      length of argument N
 *	+.......+
 *	|	|		pkt_argval[pkt_arglen] argument N
 *      +-------+
 *	
 * Note :-
 *
 *	o all numeric values are represented as 4-digit hex numbers
 *	o all offsets are byte offsets within the packet
 *
 *---------------------------------------------------------------------------*/

#define	PKT_LEN			0
#define	PKT_TYPE		PKT_LEN+8
#define		PKT_TYPE_START		'A'
#define		PKT_TYPE_STOP		'B'
#define		PKT_TYPE_CONNECT	'C'
#define		PKT_TYPE_DISCONNECT	'D'
#define		PKT_TYPE_OPEN		'E'
#define		PKT_TYPE_CLOSE		'F'
#define		PKT_TYPE_EXECSQL	'G'
#define		PKT_TYPE_COUNT		'H'
#define		PKT_TYPE_DELETE		'I'
#define		PKT_TYPE_FIND		'J'
#define		PKT_TYPE_INSERT		'K'
#define		PKT_TYPE_READ		'L'
#define		PKT_TYPE_UPDATE		'M'
#define		PKT_TYPE_MAPERROR	'O'
#define		PKT_TYPE_FETCH		'P'
#define		PKT_TYPE_FETCHMANY	'Q'
#define		PKT_TYPE_EOF		'R'
#define		PKT_TYPE_ABORT		'S'
#define		PKT_TYPE_DATA		'T'
#define		PKT_TYPE_DECLARE	'U'
#define		PKT_TYPE_UNDECLARE	'V'
#define		PKT_TYPE_METADATA	'W'
#define		PKT_TYPE_COMMIT		'X'
#define		PKT_TYPE_ROLLBACK	'Z'
#define		PKT_TYPE_ATTACH		'0'
#define		PKT_TYPE_USE		'1'
#define		PKT_TYPE_UNUSE		'2'
#define		PKT_TYPE_DETACH		'3'
#define		PKT_TYPE_LOGGING	'4'
#define		PKT_TYPE_ZAP		'5'
#define		PKT_TYPE_FIELDLIST	'6'
#define		PKT_TYPE_TABLELIST	'7'
#define		PKT_TYPE_ISLOCKED	'8'
#define		PKT_TYPE_LOCK		'9'
#define		PKT_TYPE_UNLOCK		':'
#define		PKT_TYPE_ISLOCKED_TABLE	';'
#define		PKT_TYPE_LOCK_TABLE	'<'
#define		PKT_TYPE_RLOCK_TABLE	'?'
#define		PKT_TYPE_UNLOCK_TABLE	'@'
#define		PKT_TYPE_FINDFILE	'a'
#define		PKT_TYPE_FILEATTRIBUTES	'b'
#define		PKT_TYPE_TXT_OPEN	'c'
#define		PKT_TYPE_TXT_CLOSE	'd'
#define		PKT_TYPE_TXT_READ	'e'
#define		PKT_TYPE_TXT_SEEK	'f'
#define		PKT_TYPE_TXT_WRITE	'g'
#define		PKT_TYPE_LOOKUP		'h'
#define		PKT_TYPE_NEW	        'i'
#define		PKT_TYPE_METHOD		'j'
#define		PKT_TYPE_SETPROPERTY	'k'
#define		PKT_TYPE_GETPROPERTY	'l'
#define		PKT_TYPE_ODBC_DELETEDDATA	PKT_TYPE_FIND
#define		PKT_TYPE_EXTENDED_FETCH	'm'
#define		PKT_TYPE_QUERY		'n'
#define		PKT_TYPE_MEMO		'o'
#define		PKT_TYPE_CONNECTED	'p'
#define		PKT_TYPE_MIRAGE		'q'
#define		PKT_TYPE_OK		'Y'
#define		PKT_TYPE_NOK		'N'
#define	PKT_INFO		(PKT_TYPE+1)
#define	PKT_ARGC		(PKT_INFO+8)
#define PKT_ARGV		(PKT_ARGC+4)

/*-----------------------------------------------------------*/
/* general purpose functions that handle RSI message packets */
/*-----------------------------------------------------------*/
int	rsi_getpkt();
int	rsi_getpkt_len();
int	rsi_getpkt_type();
int	rsi_getpkt_info();
int	rsi_getpkt_argc();
int	rsi_getpkt_arglen();
int	rsi_getpkt_arg();
int	rsi_getpkt_argv();
int	rsi_setup_pkt();
int	rsi_sendpkt();

/*---------------------------*/
/* client specific functions */
/*---------------------------*/
int	rsi_start();
int	rsi_stop();
int	rsi_connect();
int	rsi_disconnect();
int	rsi_execsql();
int	rsi_maperror();
int	rsi_declare();
int	rsi_undeclare();
int	rsi_open();
int	rsi_close();
int	rsi_fetch();
int	rsi_delete();
int	rsi_find();
int	rsi_update();
int	rsi_insert();
int	rsi_read();
int	rsi_count();
int	rsi_commit();
int	rsi_rollback();
int	rsi_attach();
int	rsi_detach();
int	rsi_execute();
int	rsi_use();
int	rsi_unuse();
int	rsi_zap();
int	rsi_fieldlist();
int	rsi_tablelist();
int	rsi_islocked();
int	rsi_lock();
int	rsi_unlock();
int	rsi_islocked_table();
int	rsi_lock_table();
int	rsi_rlock_table();
int	rsi_unlock_table();
int	rsi_findfile();
int	rsi_fileattributes();
int	rsi_txt_open();
int	rsi_txt_close();
int	rsi_txt_read();
int	rsi_txt_seek();
int	rsi_txt_write();
int	rsi_netobjectlookup();
int	rsi_netobject_new();
int	rsi_netobject_method();
int	rsi_netobject_getproperty();
int	rsi_netobject_setproperty();
int	rsi_readmemo();
int	rsi_mirage();

/*-----------------------*/
/* server function table */
/*-----------------------*/
struct	RSI_SERVER_TABLE {
	int	(*rsi_start)();
	int	(*rsi_stop)();
	int	(*rsi_connect)();	
	int	(*rsi_disconnect)();
	int	(*rsi_declare)();	
	int	(*rsi_undeclare)();
	int	(*rsi_open)();	
	int	(*rsi_close)();	
	int	(*rsi_execsql)();	
	int	(*rsi_count)();	
	int	(*rsi_delete)();	
	int	(*rsi_find)();	
	int	(*rsi_insert)();	
	int	(*rsi_read)();	
	int	(*rsi_update)();	
	int	(*rsi_maperror)();
	int	(*rsi_fetch)();	
	int	(*rsi_fetchmany)();
	int	(*rsi_commit)();
	int	(*rsi_rollback)();
	int	(*rsi_attach)();
	int	(*rsi_detach)();
	int	(*rsi_execute)();
	int	(*rsi_use)();
	int	(*rsi_unuse)();
	int	(*rsi_zap)();
	int	(*rsi_fieldlist)();
	int	(*rsi_tablelist)();
	int	(*rsi_islocked)();
	int	(*rsi_lock)();
	int	(*rsi_unlock)();
	int	(*rsi_islocked_table)();
	int	(*rsi_lock_table)();
	int	(*rsi_rlock_table)();
	int	(*rsi_unlock_table)();
	int	(*rsi_findfile)();
	int	(*rsi_fileattributes)();
	int	(*rsi_txt_open)();
	int	(*rsi_txt_close)();
	int	(*rsi_txt_read)();
	int	(*rsi_txt_seek)();
	int	(*rsi_txt_write)();
	int	(*rsi_netobjectlookup)();
	int	(*rsi_netobject_new)();
	int	(*rsi_netobject_method)();
	int	(*rsi_netobject_getproperty)();
	int	(*rsi_netobject_setproperty)();
	int	(*rsi_extended_fetch)();
	int	(*rsi_memo)();
	int	(*rsi_mirage)();
};


/*---------------------------*/
/* server specific functions */
/*---------------------------*/
int	rsi_server(
	char	*driver_name,
	int	argc,
	char	*argv[],
	struct	RSI_SERVER_TABLE *server_table);

/*--------------------------*/
/* define connection errors */
/*--------------------------*/
#define	RSIERR_INVALID_LICENSE		-1	/* Incorrect license for UAS version */
#define	RSIERR_INVALID_USERNAME		-2	/* Invalid username/password */
#define	RSIERR_INVALID_AUTHORATION	-3	/* Invalid license authorization code */
#define	RSIERR_LICENSE_EXPIRIED		-4	/* UAS license has expired */
#define	RSIERR_INVALID_NODE		-5	/* Incorrect license for node */
#define	RSIERR_INVALID_CLIENT		-6	/* License can not accept requests from differemt IP addresses */
#define	RSIERR_INVALID_ENVRONMENT	-7	/* Envronment variable DB_LICENSE is not defined */
#define	RSIERR_CANTOPEN_LICENSE_FILE	-8	/* Can't open license file */
#define	RSIERR_CANTREAD_LICENSE_FILE	-9	/* Can't read license file */
#define	RSIERR_GATEWAY_NOTLICENSED	-10	/* Gateway not licensed for use */
#define	RSIERR_INVALID_CLIENT_OPTIONS	-11	/* Client not licensed for option string */
#define	RSIERR_INVALID_CPU_COUNT	-12	/* The UAS is not licesed for %d CPU's */
#define	RSIERR_INVALID_PRODUCTCODE	-13	/* Incorrect license for UAS product code */
#define	RSIERR_SERVER_PAUSED		-14	/* Server is paused and not accepting connections */
#define	RSIERR_USER_COUNT_EXCEEDED	-15	/* User count exceeded */
#define	RSIERR_SERVER_CONFIG_ERROR	-16	/* Error in the config file */
#define	RSIERR_ACCESS_DENIED		-17	/* Access denied to application */

#endif  /* RSI_H */
