#include <stdio.h>
#include <sql.h>
#include <sqlext.h>
#include <malloc.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

static	int get_vardata(
	SQLHSTMT  hstmt);
static	int put_vardata(
	SQLHSTMT  hstmt);
static	void show_error(
	int	line,
        SQLSMALLINT HandleType,
	SQLHSTMT  hstmt);

#define TRUE 1
int main(int argc, char* argv[])
{
	SQLHENV  henv;
	SQLHDBC  hdbc;
	SQLHSTMT hstmt;
	SQLRETURN  retcode;

	      /*Allocate environment handle */
	retcode = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);  
	if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {
	   /* Set the ODBC version environment attribute */
	   retcode = SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0); 

	   if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {
	      /* Allocate connection handle */
	      retcode = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc); 

	      if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {
		 /* Set login timeout to 5 seconds. */
		 retcode = SQLSetConnectAttr(hdbc, SQL_LOGIN_TIMEOUT, (void*)5, 0);

		 /* Connect to data source */
		 retcode = SQLConnect(hdbc, (SQLCHAR*) "Recital ODBC test", SQL_NTS,
			(SQLCHAR*) "?", SQL_NTS,
			(SQLCHAR*) "?", SQL_NTS);

		 if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO){
		    /* Allocate statement handle */
		    retcode = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt); 

		    if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {
			/* Get Binary data */
			retcode = get_vardata(hstmt);
			if (retcode == SQL_SUCCESS) {
			    /* Put Binary data */
			    retcode = put_vardata(hstmt);
			}

		       SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
		    }
		    SQLDisconnect(hdbc);
		 } else {
	   	    show_error(__LINE__,SQL_HANDLE_DBC, hdbc);
		 }
		 SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
	      }
	   }
	} else {
	   show_error(__LINE__,SQL_HANDLE_ENV, henv);
	   SQLFreeHandle(SQL_HANDLE_ENV, henv);
	}

	return 0;
}


static	int get_vardata(SQLHSTMT  hstmt)
{
	SQLCHAR	    szName[50];
	SQLINTEGER  sCustID, cbName, cbCustID, cbDescp, cbPicture;
	SQLRETURN   retcode;
	char	    *description;
	char	    *picture;
	char	    filename[512];
	int	    cnt=1;
	int	    fd;
	int	    len;

	retcode = SQLExecDirect(hstmt,
				(unsigned char *)"SELECT CATEGORYID, CATEGORYNAME, DESCRIPTION, PICTURE FROM CATEGORIES",
				SQL_NTS);

	if (retcode == SQL_SUCCESS) {
	    SQLBindCol(hstmt, 1, SQL_C_ULONG, &sCustID, 0, &cbCustID);
	    SQLBindCol(hstmt, 2, SQL_C_CHAR, szName, 50, &cbName);
	    while (TRUE) {
		retcode = SQLFetch(hstmt);
		if (retcode == SQL_ERROR || retcode == SQL_SUCCESS_WITH_INFO) {
		    show_error(__LINE__,SQL_HANDLE_STMT, hstmt);
		}
		if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO){

		    /* Get data from VARCHAR description field */
		    retcode = SQLGetData(hstmt, 3, SQL_C_CHAR, description, 0, &cbDescp);
		    description = (char *)malloc(cbDescp+1);
		    SQLGetData(hstmt, 3, SQL_C_CHAR, description, cbDescp, &cbDescp);
		    description[cbDescp] = '\0';

		    /* Get data from VARBINARY picture field */
		    retcode = SQLGetData(hstmt, 4, SQL_C_BINARY, picture, 0, &cbPicture);
		    picture = (char *)malloc(cbPicture);
		    SQLGetData(hstmt, 4, SQL_C_BINARY, picture, cbPicture, &cbPicture);
		    sprintf(filename, "categoty%d.jpg", cnt++);
		    fd = open(filename, O_CREAT | O_RDWR);
		    len = write(fd, picture, cbPicture);
		    close(fd);
		    free(picture);

		     /* Print the row of data    */
		     fprintf(stdout, "%-5d %-*s \n%s\n", sCustID, 50-1, szName, description);
		     free(description);
		} else {
		    break;
		}
	    }
	} else {
	    show_error(__LINE__,SQL_HANDLE_STMT, hstmt);
	}
	return(retcode);
}

#define  MAX_DATA_LEN 1024
static	int put_vardata(SQLHSTMT  hstmt)
{
	SQLINTEGER cbPhotoParam, cbData, cbDesc;
	SQLPOINTER pToken;
	SQLCHAR  Data[MAX_DATA_LEN];
	SQLCHAR  sDesc[MAX_DATA_LEN];
	SQLRETURN  retcode;
	int	fd;

	retcode = SQLExecDirect(hstmt,
			     (unsigned char *)"CREATE TABLE PICTURES (partid INT AUTO_INCREMENT, description LONG VARCHAR, picture LONG VARBINARY)", 
			     SQL_NTS);

	if (retcode == SQL_ERROR) {
	    show_error(__LINE__,SQL_HANDLE_STMT, hstmt);
	    return(retcode);

	}
	retcode = SQLPrepare(hstmt,
			    (unsigned char *)"INSERT INTO PICTURES (description, picture) VALUES (?, ?)", 
			    SQL_NTS);
	if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO) {

	    /* Bind the parameters. For parameter 2, pass */
	    /* the parameter number in ParameterValuePtr instead of a buffer */    
	    /* address. */
	    strcpy((char *)sDesc, "First text");
	    cbDesc = strlen((char *)sDesc);
	    SQLBindParameter(hstmt, 
			    1, 
			    SQL_PARAM_INPUT, 
			    SQL_C_CHAR, 
			    SQL_CHAR, 
			    0, 
			    0, 
			    &sDesc, 
			    0, 
			    &cbDesc);
	    SQLBindParameter(hstmt, 
			    2, 
			    SQL_PARAM_INPUT,
			    SQL_C_BINARY, 
			    SQL_LONGVARBINARY,
			    0, 
			    0, 
			    (SQLPOINTER) 2, 
			    0, 
			    &cbPhotoParam);

	    /* Set values so data for parameter 2 will be */
	    /* passed at execution. Note that the length parameter in */
	    /* the macro SQL_LEN_DATA_AT_EXEC is 0. This assumes that */
	    /* the driver returns "N" for the SQL_NEED_LONG_DATA_LEN */
	    /* information type in SQLGetInfo.         */

	    cbPhotoParam = SQL_LEN_DATA_AT_EXEC(0);

	    retcode = SQLExecute(hstmt);

	    /* For data-at-execution parameters, call SQLParamData to */
	    /* get the parameter number set by SQLBindParameter. */
	    /* Call InitUserData. Call GetUserData and SQLPutData */
	    /* repeatedly to get and put all data for the parameter. */
	    /* Call SQLParamData to finish processing this parameter */
	    while (retcode == SQL_NEED_DATA) {
		retcode = SQLParamData(hstmt, &pToken);
		if (retcode == SQL_NEED_DATA) {
		    fd = open("category1.jpg", O_RDONLY);
		    while ((cbData = read(fd, Data, MAX_DATA_LEN)) > 0) {
			SQLPutData(hstmt, Data, cbData);
		    }
		} else	if (retcode == SQL_ERROR) {
		    show_error(__LINE__,SQL_HANDLE_STMT, hstmt);
		}
	    }
	}
	return(retcode);
}


static	void	show_error(int line, SQLSMALLINT HandleType, SQLHSTMT  hstmt)
{
	SQLINTEGER NativeErrorPtr;
	unsigned    char	Sqlstate[6];
	unsigned    char	MessageText[512];
	SQLSMALLINT TextLengthPtr;

	SQLGetDiagRec(HandleType,
		      hstmt,
		      1,
		      Sqlstate,
		      &NativeErrorPtr,
		      MessageText,
		      512,
		      &TextLengthPtr);
	fprintf(stdout, "%d:%s\n",line, MessageText);
}
